@@ -9,6 +9,7 @@ import Queue as queue
 import heapq
 import types
 import weakref
+import traceback
 import threading
 import xmlrpclib
 import itertools
@@ -192,6 +193,7 @@ class _weakfunction_ref(object):
 class Transporter(object):
     def __init__(self):
         self._followers = {}
+        self.lock = threading.RLock()
 
     def get_valid_followers(self):
         for name, followers in self._followers.items():
@@ -201,8 +203,10 @@ class Transporter(object):
                     yield id_obj, function
 
     def regist_follower(self, id_obj, function):
+        self.lock.acquire()
         followers = self._followers.setdefault(id_obj.name, [])
         followers.append((id_obj, function))
+        self.lock.release()
 
     def twitter(self, id_obj, args, kwargs, counter=100):
         self.twitter_local(id_obj, args, kwargs, counter)
@@ -224,12 +228,29 @@ class Transporter(object):
                 function()(message)
             else:
                 ThreadPool.request_work(function(), message)
+                
+    def get_matched_followers(self, id_obj):
+        followers = []
+        self.lock.acquire()
+        if id_obj.name is None:
+            actions = self._followers.values()
+        else:
+            wildcard_actions = self._followers.get(None, [])
+            certain_actions = self._followers.get(id_obj.name, [])
+            actions = [wildcard_actions, certain_actions]
+        for following_id, function in itertools.chain(*actions):
+            if not following_id.is_match(id_obj):
+                continue
+            followers.append(following_id)
+        self.lock.release()
+        return followers
 
 
 class RootTransporter(object):
     _default_namespace = None
     _namespaces = {}
     _connections = None
+    _lock = threading.RLock()
 
     @classmethod
     def twitter(cls, id_obj, args, kwargs):
@@ -246,19 +267,28 @@ class RootTransporter(object):
         cls.get(namespace).twitter(id_obj, args, kwargs)
 
     @classmethod
-    def regist_follower(cls, id_obj, method):
+    def regist_follower(cls, id_obj, callable):
         namespace = id_obj.namespace
         if namespace is None:
             namespace = cls._default_namespace
-        cls.get(namespace).regist_follower(id_obj, method)
+        cls.get(namespace).regist_follower(id_obj, callable)
 
     @classmethod
     def get(cls, namespace):
+        cls._lock.acquire()
         message_queue = cls._namespaces.get(namespace)
         if message_queue is None:
             message_queue = Transporter()
             cls._namespaces[namespace] = message_queue
+        cls._lock.release()
         return message_queue
+    
+    @classmethod
+    def get_matched_followers(cls, id_obj):
+        namespace = id_obj.namespace
+        if namespace is None:
+            namespace = cls._default_namespace
+        return cls.get(namespace).get_matched_followers(id_obj)
 
 
 class ThreadPool(object):
